/*
 * Copyright (C) Intel Corporation
 * Author: Shao Guohua <guohua.shao@intel.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 */

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/etherdevice.h>
#include <net/lantiq_cbm_api.h>
#include <net/datapath_api.h>
#include "../datapath.h"
#include "datapath_misc.h"

#if IS_ENABLED(CONFIG_LTQ_DATAPATH_DDR_SIMULATE_GSWIP31)
#include "datapath_gswip_simulate.h"
#endif

struct ctp_assign {
	u32 flag; /*Datapath Device Flag */
	GSW_LogicalPortMode_t emode; /*mapped GSWIP CTP flag */
	u16 num; /*Max CTP allowed for that GSWIP logical port*/
	u32 vap_offset; /*VAP offset */
	u32 vap_mask; /*VAP Mask after shift vap_offset bits */
	u32 lookup_mode; /*CQE lookup mode  */
};

static struct ctp_assign ctp_assign_info[] = {
	/*note: multiple flags must put first */
	{ DP_F_GPON, GSW_LOGICAL_PORT_GPON, 256, 0, 0xFF, CQE_LU_MODE1 },
	{ DP_F_EPON, GSW_LOGICAL_PORT_EPON, 256, 0, 0xFF, CQE_LU_MODE1 },
	{ DP_F_GINT, GSW_LOGICAL_PORT_GINT, 8, 0, 0xFF, CQE_LU_MODE1 },
/*#define DP_ETH_TEST*/
#ifndef DP_ETH_TEST
	{ DP_F_FAST_ETH_WAN, GSW_LOGICAL_PORT_8BIT_WLAN, 8, 8, 0xF,
	  CQE_LU_MODE2 },
	{ DP_F_FAST_ETH_LAN | DP_F_ALLOC_EXPLICIT_SUBIFID,
	  GSW_LOGICAL_PORT_8BIT_WLAN, 8, 8, 0xF, CQE_LU_MODE2 },
	{ DP_F_FAST_ETH_LAN, GSW_LOGICAL_PORT_8BIT_WLAN, 2, 8, 0xF,
	  CQE_LU_MODE2 },
#else /*testing only */
	{ DP_F_FAST_ETH_WAN, GSW_LOGICAL_PORT_OTHER, 1, 8, 0xF, CQE_LU_MODE2 },
	{ DP_F_FAST_ETH_LAN | DP_F_ALLOC_EXPLICIT_SUBIFID,
	  GSW_LOGICAL_PORT_OTHER, 1, 8, 0xF, CQE_LU_MODE2 },
	{ DP_F_FAST_ETH_LAN, GSW_LOGICAL_PORT_OTHER, 1, 8, 0xF, CQE_LU_MODE2 },
#endif
	{ DP_F_FAST_WLAN, GSW_LOGICAL_PORT_8BIT_WLAN, 16, 8, 0xF, CQE_LU_MODE2 },
	{ DP_F_FAST_WLAN_EXT, GSW_LOGICAL_PORT_9BIT_WLAN, 8, 9, 0x7,
	  CQE_LU_MODE2 }
};

static struct ctp_assign ctp_assign_def = { 0, GSW_LOGICAL_PORT_8BIT_WLAN, 8, 8,
					    0xF, CQE_LU_MODE2 };

static struct gsw_itf itf_assign[PMAC_MAX_NUM] = { 0 };

static char *ctp_mode_string(GSW_LogicalPortMode_t type)
{
	if (type == GSW_LOGICAL_PORT_8BIT_WLAN)
		return "8BIT_WLAN";
	if (type == GSW_LOGICAL_PORT_9BIT_WLAN)
		return "9BIT_WLAN";
	if (type == GSW_LOGICAL_PORT_GPON)
		return "GPON";
	if (type == GSW_LOGICAL_PORT_EPON)
		return "EPON";
	if (type == GSW_LOGICAL_PORT_GINT)
		return "GINT";
	if (type == GSW_LOGICAL_PORT_OTHER)
		return "OTHER";
	return "Undef";
}

#if IS_ENABLED(CONFIG_LTQ_DATAPATH_DDR_SIMULATE_GSWIP31)
GSW_return_t gsw_core_api_ddr_simu31(dp_gsw_cb func, void *ops, void *param)
{
	if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_brdgport_ops.BridgePort_Alloc) {
		return BridgePortAlloc(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_brdgport_ops.BridgePort_ConfigGet) {
		return BridgePortConfigGet(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_brdgport_ops.BridgePort_ConfigSet) {
		return BridgePortConfigSet(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_brdgport_ops.BridgePort_Free) {
		return BridgePortFree(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_brdg_ops.Bridge_Alloc) {
		return BridgeAlloc(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_brdg_ops.Bridge_ConfigSet) {
		return BridgeConfigSet(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_brdg_ops.Bridge_ConfigGet) {
		return BridgeConfigGet(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_brdg_ops.Bridge_Free) {
		return BridgeFree(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_ctp_ops.CTP_PortAssignmentAlloc) {
		return CTP_PortAssignmentAlloc(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_ctp_ops.CTP_PortAssignmentFree) {
		return CTP_PortAssignmentFree(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_ctp_ops.CTP_PortAssignmentSet) {
		return CTP_PortAssignmentSet(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_ctp_ops.CTP_PortAssignmentGet) {
		return CTP_PortAssignmentGet(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_ctp_ops.CTP_PortConfigSet) {
		return CtpPortConfigSet(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_ctp_ops.CTP_PortConfigGet) {
		return CtpPortConfigGet(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_swmac_ops.MAC_TableEntryAdd) {
		return MacTableAdd(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_swmac_ops.MAC_TableEntryRead) {
		return MacTableRead(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_swmac_ops.MAC_TableEntryRemove) {
		return MacTableRemove(param);
	} else if (func == (dp_gsw_cb)dp_port_prop[0].ops[0]->gsw_swmac_ops.MAC_TableEntryQuery) {
		return MacTableQuery(param);
	}
	return GSW_SIMUTE_DDR_NOT_MATCH;
}
#endif

/*This API is only for GSWIP-R PMAC modification, not for GSWIP-L */
int dp_pmac_set_31(int inst, u32 port, dp_pmac_cfg_t *pmac_cfg)
{
	GSW_PMAC_Eg_Cfg_t egcfg;
	GSW_PMAC_Ig_Cfg_t igcfg;
	int i, j, k;
	u32 flag = 0;
	cbm_dq_port_res_t dqport;
	s32 ret;
	u32 pmacid;
	struct core_ops *gswr_r;
	GSW_PMAC_Glbl_Cfg_t pmac_glb;

	if (!pmac_cfg || !port) {
		PR_ERR("dp_pmac_set:wrong parameter(pmac_cfg/port NULL)\n");
		return -1;
	}

	if (!pmac_cfg->ig_pmac_flags && !pmac_cfg->eg_pmac_flags)
		return 0;
	if (port == 2)
		pmacid = 1;
	else
		pmacid = 0;
	memset(&dqport, 0, sizeof(cbm_dq_port_res_t));

	/* Get GSWIP device handler */
	gswr_r = dp_port_prop[inst].ops[0];

	/*set ingress port via DMA tx channel */
	if (pmac_cfg->ig_pmac_flags) {
		/*Read back igcfg from gsw first */
		ret = cbm_dequeue_port_resources_get(port, &dqport, flag);

		if (ret == -1) {
			PR_ERR("cbm_dequeue_port_resources_get failed\n");
			return -1;
		}

		memset(&igcfg, 0, sizeof(GSW_PMAC_Ig_Cfg_t));

		for (i = 0; i < dqport.num_deq_ports; i++) {
			igcfg.nPmacId = pmacid;
			igcfg.nTxDmaChanId = dqport.deq_info[i].dma_tx_chan;
			if (igcfg.nTxDmaChanId == (u8)-1) {
				igcfg.nTxDmaChanId =
					pmac_cfg->ig_pmac.tx_dma_chan;
			}
			gsw_core_api((dp_gsw_cb)gswr_r->gsw_pmac_ops.Pmac_Ig_CfgGet, gswr_r, &igcfg);

			/*update igcfg and write back to gsw */
			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_ERR_DISC)
				igcfg.bErrPktsDisc =
					pmac_cfg->ig_pmac.err_disc;

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PRESENT)
				igcfg.bPmacPresent = pmac_cfg->ig_pmac.pmac;

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_SUBIF)
				//igcfg.bSubIdDefault =
				igcfg.eSubId =
					pmac_cfg->ig_pmac.def_pmac_subifid;

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_SPID)
				igcfg.bSpIdDefault =
					pmac_cfg->ig_pmac.def_pmac_src_port;

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_CLASSENA)
				igcfg.bClassEna =
					pmac_cfg->ig_pmac.def_pmac_en_tc;

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_CLASS)
				igcfg.bClassDefault =
					pmac_cfg->ig_pmac.def_pmac_tc;

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PMAPENA)
				igcfg.bPmapEna =
					pmac_cfg->ig_pmac.def_pmac_en_pmap;

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PMAP)
				igcfg.bPmapDefault =
					pmac_cfg->ig_pmac.def_pmac_pmap;

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PMACHDR1)
				igcfg.defPmacHdr[0] =
					pmac_cfg->ig_pmac.def_pmac_hdr[0];

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PMACHDR2)
				igcfg.defPmacHdr[1] =
					pmac_cfg->ig_pmac.def_pmac_hdr[1];

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PMACHDR3)
				igcfg.defPmacHdr[2] =
					pmac_cfg->ig_pmac.def_pmac_hdr[2];

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PMACHDR4)
				igcfg.defPmacHdr[3] =
					pmac_cfg->ig_pmac.def_pmac_hdr[3];

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PMACHDR5)
				igcfg.defPmacHdr[4] =
					pmac_cfg->ig_pmac.def_pmac_hdr[4];

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PMACHDR6)
				igcfg.defPmacHdr[5] =
					pmac_cfg->ig_pmac.def_pmac_hdr[5];

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PMACHDR7)
				igcfg.defPmacHdr[6] =
					pmac_cfg->ig_pmac.def_pmac_hdr[6];

			if (pmac_cfg->ig_pmac_flags & IG_PMAC_F_PMACHDR8)
				igcfg.defPmacHdr[7] =
					pmac_cfg->ig_pmac.def_pmac_hdr[7];

			DP_DEBUG(DP_DBG_FLAG_DBG,
				 "\nPMAC %d igcfg configuration:\n", port);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.nPmacId=%d\n",
				 igcfg.nPmacId);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.nTxDmaChanId=%d\n",
				 igcfg.nTxDmaChanId);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.bErrPktsDisc=%d\n",
				 igcfg.bErrPktsDisc);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.bPmapDefault=%d\n",
				 igcfg.bPmapDefault);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.bPmapEna=%d\n",
				 igcfg.bPmapEna);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.bClassDefault=%d\n",
				 igcfg.bClassDefault);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.bClassEna=%d\n",
				 igcfg.bClassEna);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.bSubIdDefault=%d\n",
				 igcfg.eSubId);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.bSpIdDefault=%d\n",
				 igcfg.bSpIdDefault);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.bPmacPresent=%d\n",
				 igcfg.bPmacPresent);
			DP_DEBUG(DP_DBG_FLAG_DBG, "igcfg.defPmacHdr=");

			for (k = 0;
			     k <
			     sizeof(igcfg.defPmacHdr) /
				     sizeof(igcfg.defPmacHdr[0]);
			     k++)
				DP_DEBUG(DP_DBG_FLAG_DBG, "0x%x ",
					 igcfg.defPmacHdr[k]);

			DP_DEBUG(DP_DBG_FLAG_DBG, "\n");

			gsw_core_api((dp_gsw_cb)gswr_r->gsw_pmac_ops
					     .Pmac_Ig_CfgSet,
				     gswr_r, &igcfg);
		}

		kfree(dqport.deq_info);
	}

	/*set egress port via pmac port id */
	if (!pmac_cfg->eg_pmac_flags)
		return 0;

	for (i = 0; i <= 15; i++) { /*traffic class */
		for (j = 0; j <= 3; j++) { /* flow */
			/*read back egcfg first from gsw */
			memset(&egcfg, 0, sizeof(GSW_PMAC_Eg_Cfg_t));
			egcfg.nPmacId = pmacid;
			egcfg.nDestPortId = port;
			egcfg.nTrafficClass = i;
			egcfg.nFlowIDMsb = j;
			egcfg.nBslTrafficClass = i;

			memset(&pmac_glb, 0, sizeof(pmac_glb));
			gsw_core_api((dp_gsw_cb)gswr_r->gsw_pmac_ops
					     .Pmac_Gbl_CfgGet,
				     gswr_r, &pmac_glb);
			egcfg.bProcFlagsSelect = pmac_glb.bProcFlagsEgCfgEna;
			DP_DEBUG(DP_DBG_FLAG_DBG, "bProcFlagsSelect=%u\n",
				 egcfg.bProcFlagsSelect);

			/*update egcfg and write back to gsw */
			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_FCS)
				egcfg.bFcsEna = pmac_cfg->eg_pmac.fcs;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_L2HDR_RM) {
				egcfg.bRemL2Hdr = pmac_cfg->eg_pmac.rm_l2hdr;
				egcfg.numBytesRem =
					pmac_cfg->eg_pmac.num_l2hdr_bytes_rm;
			}

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_PMAC)
				egcfg.bPmacEna = pmac_cfg->eg_pmac.pmac;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_RXID)
				egcfg.nRxDmaChanId =
					pmac_cfg->eg_pmac.rx_dma_chan;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_REDIREN)
				egcfg.bRedirEnable = pmac_cfg->eg_pmac.redir;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_BSLSEG)
				egcfg.bBslSegmentDisable =
					pmac_cfg->eg_pmac.bsl_seg;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_RESDW1EN)
				egcfg.bResDW1Enable =
					pmac_cfg->eg_pmac.res_endw1;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_RESDW1)
				egcfg.nResDW1 = pmac_cfg->eg_pmac.res_dw1;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_RES1DW0EN)
				egcfg.bRes1DW0Enable =
					pmac_cfg->eg_pmac.res1_endw0;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_RES1DW0)
				egcfg.nRes1DW0 = pmac_cfg->eg_pmac.res1_dw0;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_RES2DW0EN)
				egcfg.bRes2DW0Enable =
					pmac_cfg->eg_pmac.res2_endw0;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_RES2DW0)
				egcfg.nRes2DW0 = pmac_cfg->eg_pmac.res2_dw0;

			/*if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_TCENA)
			 *    egcfg.bTCEnable = pmac_cfg->eg_pmac.tc_enable;
			 */

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_DECFLG)
				egcfg.bDecFlag = pmac_cfg->eg_pmac.dec_flag;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_ENCFLG)
				egcfg.bEncFlag = pmac_cfg->eg_pmac.enc_flag;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_MPE1FLG)
				egcfg.bMpe1Flag = pmac_cfg->eg_pmac.mpe1_flag;

			if (pmac_cfg->eg_pmac_flags & EG_PMAC_F_MPE2FLG)
				egcfg.bMpe2Flag = pmac_cfg->eg_pmac.mpe2_flag;
#if defined(CONFIG_LTQ_DATAPATH_DBG) && CONFIG_LTQ_DATAPATH_DBG
			if (dp_dbg_flag) {
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "\nPMAC %d egcfg configuration:\n",
					 port);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.nPmacId=%d\n",
					 egcfg.nPmacId);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.nRxDmaChanId=%d\n",
					 egcfg.nRxDmaChanId);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bRemL2Hdr=%d\n",
					 egcfg.bRemL2Hdr);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.numBytesRem=%d\n",
					 egcfg.numBytesRem);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bFcsEna=%d\n", egcfg.bFcsEna);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bPmacEna=%d\n",
					 egcfg.bPmacEna);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bResDW1Ena=%d\n",
					 egcfg.bResDW1Enable);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.nResDW1=%d\n", egcfg.nResDW1);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bRes1DW0Ena=%d\n",
					 egcfg.bRes1DW0Enable);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.nRes1DW0=%d\n",
					 egcfg.nRes1DW0);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bRes2DW0Ena=%d\n",
					 egcfg.bRes2DW0Enable);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.nRes2DW0=%d\n",
					 egcfg.nRes2DW0);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.nDestPortId=%d\n",
					 egcfg.nDestPortId);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bTCEnable=%d\n",
					 egcfg.bTCEnable);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.nTrafficClass=%d\n",
					 egcfg.nTrafficClass);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.nBslTrafficClass=%d\n",
					 egcfg.nBslTrafficClass);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.nFlowIDMsb=%d\n",
					 egcfg.nFlowIDMsb);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bDecFlag=%d\n",
					 egcfg.bDecFlag);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bEncFlag=%d\n",
					 egcfg.bEncFlag);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bMpe1Flag=%d\n",
					 egcfg.bMpe1Flag);
				DP_DEBUG(DP_DBG_FLAG_DBG,
					 "egcfg.bMpe2Flag=%d\n",
					 egcfg.bMpe2Flag);
			}
#endif
			gsw_core_api((dp_gsw_cb)gswr_r->gsw_pmac_ops
					     .Pmac_Eg_CfgSet,
				     gswr_r, &egcfg);
			;
		}
	}

	return 0;
}

/*flag: bit 0 for cpu
 * bit 1 for mpe1,bit 2 for mpe2, bit 3 for mpe3;
 */
#define GSW_L_BASE_ADDR (0xBC000000)
#define GSW_R_BASE_ADDR (0xBA000000)
#define FDMA_PASR_ADDR (0xA47)

int dp_set_gsw_parser_31(u8 flag, u8 cpu, u8 mpe1,
			 u8 mpe2, u8 mpe3)
{
	GSW_CPU_PortCfg_t param = { 0 };
	struct core_ops *gsw_handle = dp_port_prop[0].ops[0]; /*gswip o */

	if (gsw_core_api((dp_gsw_cb)gsw_handle->gsw_common_ops
				 .CPU_PortCfgGet,
			 gsw_handle, &param)) {
		PR_ERR("Failed GSW_CPU_PORT_CFG_GET\n");
		return -1;
	}
	DP_DEBUG(DP_DBG_FLAG_DBG, "old flag=0x%x cpu=%d mpe1/2/3=%d/%d/%d\n",
		 flag, param.eNoMPEParserCfg,
		 param.eMPE1ParserCfg, param.eMPE2ParserCfg,
		 param.eMPE1MPE2ParserCfg);
	DP_DEBUG(DP_DBG_FLAG_DBG, "new flag=0x%x cpu=%d mpe1/2/3=%d/%d/%d\n",
		 flag, cpu, mpe1, mpe2, mpe3);
	if (flag & F_MPE_NONE)
		param.eNoMPEParserCfg = cpu;

	if (flag & F_MPE1_ONLY)
		param.eMPE1ParserCfg = mpe1;

	if (flag & F_MPE2_ONLY)
		param.eMPE2ParserCfg = mpe2;

	if (flag & F_MPE1_MPE2)
		param.eMPE1MPE2ParserCfg = mpe3;

	if (gsw_core_api((dp_gsw_cb)gsw_handle->gsw_common_ops
				 .CPU_PortCfgSet,
			 gsw_handle, &param)) {
		PR_ERR("Failed GSW_CPU_PORT_CFG_SET\n");
		return -1;
	}
	dp_parser_info_refresh(param.eNoMPEParserCfg,
			       param.eMPE1ParserCfg,
			       param.eMPE2ParserCfg,
			       param.eMPE1MPE2ParserCfg, 0);
	return 0;
}

int dp_get_gsw_parser_31(u8 *cpu, u8 *mpe1, u8 *mpe2,
			 u8 *mpe3)
{
	GSW_CPU_PortCfg_t param = { 0 };
	struct core_ops *gsw_handle = dp_port_prop[0].ops[0]; /*gswip 0*/

	if (gsw_core_api((dp_gsw_cb)gsw_handle->gsw_common_ops
				 .CPU_PortCfgGet,
			 gsw_handle, &param)) {
		PR_ERR("Failed GSW_CPU_PORT_CFG_GET\n");
		return -1;
	}
	dp_parser_info_refresh(param.eNoMPEParserCfg,
			       param.eMPE1ParserCfg,
			       param.eMPE2ParserCfg,
			       param.eMPE1MPE2ParserCfg, 1);

	if (cpu) {
		*cpu = param.eNoMPEParserCfg;
		DP_DEBUG(DP_DBG_FLAG_DBG, "  cpu=%d\n", *cpu);
	}

	if (mpe1) {
		*mpe1 = param.eMPE1ParserCfg;
		DP_DEBUG(DP_DBG_FLAG_DBG, "  mpe1=%d\n", *mpe1);
	}

	if (mpe2) {
		*mpe2 = param.eMPE2ParserCfg;
		DP_DEBUG(DP_DBG_FLAG_DBG, "  mpe2=%d\n", *mpe2);
	}

	if (mpe3) {
		*mpe3 = param.eMPE1MPE2ParserCfg;
		DP_DEBUG(DP_DBG_FLAG_DBG, "  mpe3=%d\n", *mpe3);
	}
	return 0;
}

int gsw_mib_reset_31(int dev, u32 flag)
{
	struct core_ops *gsw_handle;
	GSW_return_t ret;
	GSW_RMON_clear_t rmon_clear;

	gsw_handle = dp_port_prop[0].ops[0];
	rmon_clear.eRmonType = GSW_RMON_ALL_TYPE;
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_rmon_ops.RMON_Clear,
			   gsw_handle, &rmon_clear);

	if (ret != GSW_statusOk) {
		PR_ERR("R:GSW_RMON_CLEAR failed for GSW_RMON_ALL_TYPE\n");
		return -1;
	}
	return ret;
}

/* Return allocated ctp number */
struct gsw_itf *ctp_port_assign(int inst, u8 ep, int bp_default,
				u32 flags, struct dp_dev_data *data)
{
	GSW_CTP_portAssignment_t ctp_assign;
	struct ctp_assign *assign = &ctp_assign_def;
	int i, alloc_flag;
	u16 num;
	struct core_ops *gsw_handle;

	memset(&ctp_assign, 0, sizeof(ctp_assign));
	gsw_handle = dp_port_prop[inst].ops[GSWIP_L];
	alloc_flag = dp_port_info[inst][ep].alloc_flags;

	if (flags & DP_F_DEREGISTER) {
		PR_ERR("Need to Free CTP Port here for ep=%d\n", ep);
		ctp_assign.nLogicalPortId = ep;
		ctp_assign.eMode = itf_assign[ep].mode;
		ctp_assign.nFirstCtpPortId = itf_assign[ep].start;
		ctp_assign.nNumberOfCtpPort = itf_assign[ep].n;
		if (gsw_core_api((dp_gsw_cb)gsw_handle->gsw_ctp_ops
					 .CTP_PortAssignmentFree,
				 gsw_handle,
				 &ctp_assign) != 0) {
			PR_ERR("Failed to allc CTP for ep=%d blk=%d mode=%d\n",
			       ep, assign->num, assign->emode);
			return NULL;
		}
		return NULL;
	}

	for (i = 0; i < ARRAY_SIZE(ctp_assign_info); i++) {
		if ((ctp_assign_info[i].flag & alloc_flag) ==
		    ctp_assign_info[i].flag) {
			assign = &ctp_assign_info[i];
			break;
		}
	}
	if (data->max_ctp)
		num = data->max_ctp;
	else
		num = assign->num;

	ctp_assign.nLogicalPortId = ep;
	ctp_assign.eMode = assign->emode;
	ctp_assign.nBridgePortId = bp_default;
	ctp_assign.nFirstCtpPortId = 0;
	ctp_assign.nNumberOfCtpPort = num;
	if (gsw_core_api((dp_gsw_cb)gsw_handle->gsw_ctp_ops
				 .CTP_PortAssignmentAlloc,
			 gsw_handle,
			 &ctp_assign) != 0) {
		PR_ERR("Failed CTP Assignment for ep=%d blk size=%d mode=%s\n",
		       ep, num, ctp_mode_string(assign->emode));
		return NULL;
	}

	DP_DEBUG(DP_DBG_FLAG_DBG, "assign ep=%d with eMode=%d ctp_max:%d\n",
		 ep, assign->emode, ctp_assign.nNumberOfCtpPort);
	itf_assign[ep].mode = assign->emode;
	itf_assign[ep].n = ctp_assign.nNumberOfCtpPort;
	itf_assign[ep].start = ctp_assign.nFirstCtpPortId;
	itf_assign[ep].end = ctp_assign.nFirstCtpPortId +
			     ctp_assign.nNumberOfCtpPort - 1;
	itf_assign[ep].ep = ep;
	dp_port_info[inst][ep].ctp_max = ctp_assign.nNumberOfCtpPort;
	dp_port_info[inst][ep].vap_offset = assign->vap_offset;
	dp_port_info[inst][ep].vap_mask = assign->vap_mask;
	return &itf_assign[ep];
}

int set_port_lookup_mode(int inst, u8 ep, u32 flags)
{
	int i, alloc_flag;
	struct ctp_assign *assign = &ctp_assign_def;

	alloc_flag = dp_port_info[inst][ep].alloc_flags;
	for (i = 0; i < ARRAY_SIZE(ctp_assign_info); i++) {
		if ((ctp_assign_info[i].flag & alloc_flag) ==
		    ctp_assign_info[i].flag) {
			assign = &ctp_assign_info[i];
			break;
		}
	}
	dp_port_info[inst][ep].cqe_lu_mode = assign->lookup_mode;
	dp_port_info[inst][ep].gsw_mode = (u32)assign->emode;
	return 0;
}

/*Allocate a bridge port with specified FID and hardcoded CPU port member */
int alloc_bridge_port(int inst, int port_id, int subif_ix,
		      int fid, int bp_member)
{
	GSW_BRIDGE_portAlloc_t bp = { 0 };
	GSW_BRIDGE_portConfig_t bp_cfg = { 0 };
	struct core_ops *gsw_handle;
	GSW_return_t ret;

	gsw_handle = dp_port_prop[inst].ops[GSWIP_L];
	/*allocate a free bridge port */
	memset(&bp, 0, sizeof(bp));
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				   .BridgePort_Alloc,
			   gsw_handle, &bp);
	if ((ret != GSW_statusOk) ||
	    (bp.nBridgePortId < 0)) {
		PR_ERR("Failed to get a bridge port\n");
		return -1;
	}
	/*set this new bridge port with specified bridge ID(fid)
	 *and bridge port map
	 */
	bp_cfg.nBridgePortId = bp.nBridgePortId;
	bp_cfg.nDestLogicalPortId = port_id;
	bp_cfg.nDestSubIfIdGroup = subif_ix;
	/* By default Disable src mac learning for registered
	 * non CPU bridge port with DP
	 */
	bp_cfg.bSrcMacLearningDisable = 1;
	bp_cfg.eMask = GSW_BRIDGE_PORT_CONFIG_MASK_BRIDGE_ID |
		       GSW_BRIDGE_PORT_CONFIG_MASK_BRIDGE_PORT_MAP |
		       GSW_BRIDGE_PORT_CONFIG_MASK_MC_SRC_MAC_LEARNING |
		       GSW_BRIDGE_PORT_CONFIG_MASK_EGRESS_CTP_MAPPING;
	bp_cfg.nBridgeId = fid;

	SET_BP_MAP(bp_cfg.nBridgePortMap, bp_member); /*CPU*/
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				   .BridgePort_ConfigSet,
			   gsw_handle, &bp_cfg);
	if (ret != GSW_statusOk) {
		PR_ERR("Failed to set bridge id(%d) and port map for bp= %d\n",
		       fid, bp_cfg.nBridgePortId);
		gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				     .BridgePort_Free,
			     gsw_handle, &bp);
		return -1;
	}

	/*ADD this bridge port to CPU bridge port's member.
	 *Need read back first
	 */
	bp_cfg.nBridgePortId = bp_member;
	bp_cfg.eMask = GSW_BRIDGE_PORT_CONFIG_MASK_BRIDGE_PORT_MAP;
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				   .BridgePort_ConfigGet,
			   gsw_handle, &bp_cfg);
	if (ret != GSW_statusOk) {
		PR_ERR("Failed to get bridge port's member for bridgeport=%d\n",
		       bp_cfg.nBridgePortId);
		gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				     .BridgePort_Free,
			     gsw_handle, &bp);
		return -1;
	}
	SET_BP_MAP(bp_cfg.nBridgePortMap, bp.nBridgePortId);
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				   .BridgePort_ConfigSet,
			   gsw_handle, &bp_cfg);
	if (ret != GSW_statusOk) {
		PR_ERR("Failed to set bridge port's member for bridgeport=%d\n",
		       bp_cfg.nBridgePortId);
		gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				     .BridgePort_Free,
			     gsw_handle, &bp);
		return -1;
	}

	return bp.nBridgePortId;
}

/*Free one GSWIP bridge port
 *First read out its port member
 *according to this port memeber, from this deleing bridge port
 *from its member's member Free this bridge port
 */
int free_bridge_port(int inst, int bp)
{
	GSW_BRIDGE_portConfig_t *tmp = NULL, *tmp2 = NULL;
	int i, j;
	GSW_return_t ret;
	struct core_ops *gsw_handle;

	gsw_handle = dp_port_prop[inst].ops[GSWIP_L];
	if (bp == CPU_BP)
		return 0;

	tmp = kzalloc(sizeof(*tmp), GFP_KERNEL);
	tmp2 = kzalloc(sizeof(*tmp2), GFP_KERNEL);
	if (!tmp || !tmp2)
		goto FREE_EXIT;

	/*read out this delting bridge port's member*/
	tmp->nBridgePortId = bp;
	tmp->eMask = GSW_BRIDGE_PORT_CONFIG_MASK_BRIDGE_PORT_MAP;
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				   .BridgePort_ConfigGet,
			   gsw_handle, tmp);
	if (ret != GSW_statusOk) {
		PR_ERR("Failed GSW_BRIDGE_PORT_CONFIG_GET: %d\n", bp);
		goto EXIT;
	}
	/*remove this delting bridgeport from other bridge port's member*/
	for (i = 0; i < ARRAY_SIZE(tmp->nBridgePortMap); i++) {
		for (j = 0; j < 16 /*u16*/; j++) {
			if (!(tmp->nBridgePortMap[i] & (1 << j)))
				continue; /*not memmber bit set */
			memset(tmp2->nBridgePortMap, 0,
			       sizeof(tmp2->nBridgePortMap));
			tmp2->eMask =
				GSW_BRIDGE_PORT_CONFIG_MASK_BRIDGE_PORT_MAP;
			tmp2->nBridgePortId = i * 16 + j;
			ret = gsw_core_api((dp_gsw_cb)gsw_handle
						   ->gsw_brdgport_ops
						   .BridgePort_ConfigGet,
					   gsw_handle,
					   tmp2);
			if (ret != GSW_statusOk) {
				PR_ERR("Failed GSW_BRIDGE_PORT_CONFIG_GET\n");
				goto EXIT;
			}
			UNSET_BP_MAP(tmp2->nBridgePortMap, bp);
			ret = gsw_core_api((dp_gsw_cb)gsw_handle
						   ->gsw_brdgport_ops
						   .BridgePort_ConfigSet,
					   gsw_handle,
					   tmp2);
			if (ret != GSW_statusOk) {
				PR_ERR("Failed GSW_BRIDGE_PORT_CONFIG_SET\n");
				goto EXIT;
			}
		}
	}
EXIT:
	/*FRee thie bridge port */
	memset(tmp, 0, sizeof(tmp));
	tmp->nBridgePortId = bp;
	tmp->eMask = GSW_BRIDGE_PORT_CONFIG_MASK_BRIDGE_PORT_MAP;
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				   .BridgePort_Free,
			   gsw_handle, tmp);
	if (ret != GSW_statusOk)
		PR_ERR("Failed to GSW_BRIDGE_PORT_FREE:%d\n", bp);
FREE_EXIT:
	kfree(tmp);
	kfree(tmp2);
	return 0;
}

int dp_gswip_mac_entry_add(int bport, int fid, int inst, u8 *addr)
{
	GSW_MAC_tableAdd_t tmp;
	GSW_return_t ret;
	struct core_ops *gsw_handle;

	gsw_handle = dp_port_prop[inst].ops[GSWIP_L];
	memset(&tmp, 0, sizeof(tmp));
	tmp.bStaticEntry = 1;
	tmp.nFId = fid;
	tmp.nPortId = bport;
	SET_BP_MAP(tmp.nPortMap, bport);
	tmp.nSubIfId = 0;
	memcpy(tmp.nMAC, addr, GSW_MAC_ADDR_LEN);
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_swmac_ops.MAC_TableEntryAdd, gsw_handle, &tmp);
	if (ret != GSW_statusOk) {
		PR_ERR("fail in setting MAC table static add entry\r\n");
		return -1;
	}
	return 0;
}

int dp_gswip_mac_entry_del(int bport, int fid, int inst, u8 *addr)
{
	GSW_MAC_tableRemove_t tmp;
	GSW_MAC_tableQuery_t mac_query;
	GSW_return_t ret;
	struct core_ops *gsw_handle;

	gsw_handle = dp_port_prop[inst].ops[GSWIP_L];
	memset(&tmp, 0, sizeof(tmp));
	memset(&mac_query, 0, sizeof(mac_query));
	mac_query.nFId = fid;
	memcpy(mac_query.nMAC, addr, GSW_MAC_ADDR_LEN);
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_swmac_ops.MAC_TableEntryQuery, gsw_handle, &tmp);
	if (ret != GSW_statusOk) {
		PR_ERR("fail in getting MAC query entry\r\n");
		return -1;
	}
	tmp.nFId = fid;
	memcpy(tmp.nMAC, addr, GSW_MAC_ADDR_LEN);
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_swmac_ops.MAC_TableEntryRemove, gsw_handle, &tmp);
	if (ret != GSW_statusOk) {
		PR_ERR("fail in setting MAC static entry remove\r\n");
		return -1;
	}
	return 0;
}

int cpu_vlan_mod_dis(int inst)
{
	GSW_QoS_portRemarkingCfg_t cfg = { 0 };
	struct core_ops *ops;
	int ret;

	ops = dp_port_prop[inst].ops[GSWIP_L];

	cfg.nPortId = 0;
	ret = gsw_core_api((dp_gsw_cb)ops->gsw_qos_ops.QoS_PortRemarkingCfgGet, ops, &cfg);
	if (ret != GSW_statusOk) {
		PR_ERR("QoS_PortRemarkingCfgGet failed\n");
		return -1;
	}

	cfg.bPCP_EgressRemarkingEnable = LTQ_FALSE;
	ret = gsw_core_api((dp_gsw_cb)ops->gsw_qos_ops.QoS_PortRemarkingCfgSet, ops, &cfg);
	if (ret != GSW_statusOk) {
		PR_ERR("QoS_PortRemarkingCfgSet failed\n");
		return -1;
	}

	return 0;
}

int dp_set_gsw_pmapper_31(int inst, int bport, int lport,
			  struct dp_pmapper *mapper, u32 flag)
{
	GSW_BRIDGE_portConfig_t bp_cfg = { 0 };
	struct core_ops *gsw_handle;
	GSW_return_t ret;
	int i, index;
	int ctp;
	struct pmac_port_info *port_info = &dp_port_info[inst][lport];

	gsw_handle = dp_port_prop[inst].ops[GSWIP_L];

	DP_DEBUG(DP_DBG_FLAG_DBG,
		 "set pmapper bport %d inst %d lport %d\n",
		 bport, inst, lport);
	bp_cfg.nBridgePortId = bport;
	bp_cfg.nDestLogicalPortId = lport;
	bp_cfg.bPmapperEnable = 1;
	bp_cfg.ePmapperMappingMode = mapper->mode;

	/* copy the sub if information in all pmapper list*/
	if (mapper->def_ctp != DP_PMAPPER_DISCARD_CTP) {
		ctp = GET_VAP(mapper->def_ctp, port_info->vap_offset,
			      port_info->vap_mask);
	} else {
		ctp = PMAPPER_DISC_CTP;
	}
	bp_cfg.sPmapper.nDestSubIfIdGroup[0] = ctp;

	for (i = 0; i < DP_PMAP_PCP_NUM; i++) {
		if (mapper->pcp_map[i] != DP_PMAPPER_DISCARD_CTP) {
			ctp = GET_VAP(mapper->pcp_map[i],
				      port_info->vap_offset,
				      port_info->vap_mask);
		} else {
			ctp = PMAPPER_DISC_CTP;
		}
		bp_cfg.sPmapper.nDestSubIfIdGroup[1 + i] = ctp;
	}
	for (i = 0; i < DP_PMAP_DSCP_NUM; i++) {
		if (mapper->dscp_map[i] != DP_PMAPPER_DISCARD_CTP) {
			ctp = GET_VAP(mapper->dscp_map[i],
				      port_info->vap_offset,
				      port_info->vap_mask);
		} else {
			ctp = PMAPPER_DISC_CTP;
		}
		index = 1 + DP_PMAP_PCP_NUM + i;
		bp_cfg.sPmapper.nDestSubIfIdGroup[index] = ctp;
	}
	bp_cfg.eMask = GSW_BRIDGE_PORT_CONFIG_MASK_EGRESS_CTP_MAPPING;

	DP_DEBUG(DP_DBG_FLAG_DBG,
		 "call switch api mode %d enable %d eMask 0x%x\n",
		 bp_cfg.ePmapperMappingMode, bp_cfg.bPmapperEnable,
		 bp_cfg.eMask);
	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				   .BridgePort_ConfigSet,
			   gsw_handle, &bp_cfg);
	if (ret != GSW_statusOk) {
		PR_ERR("fail in setting pmapper\r\n");
		return -1;
	}
	return 0;
}

int dp_get_gsw_pmapper_31(int inst, int bport, int lport,
			  struct dp_pmapper *mapper, u32 flag)
{
	GSW_BRIDGE_portConfig_t bp_cfg = { 0 };
	struct core_ops *gsw_handle;
	GSW_return_t ret;
	int i, index;
	struct hal_priv *priv;
	u16 dest;
	struct pmac_port_info *info = &dp_port_info[inst][lport];

	priv = (struct hal_priv *)dp_port_prop[inst].priv_hal;
	gsw_handle = dp_port_prop[inst].ops[GSWIP_L];

	DP_DEBUG(DP_DBG_FLAG_DBG,
		 "get bport %d inst %d lport %d\n",
		 bport, inst, lport);
	bp_cfg.nBridgePortId = bport;
	bp_cfg.nDestLogicalPortId = lport;
	bp_cfg.eMask = GSW_BRIDGE_PORT_CONFIG_MASK_EGRESS_CTP_MAPPING;

	ret = gsw_core_api((dp_gsw_cb)gsw_handle->gsw_brdgport_ops
				   .BridgePort_ConfigGet,
			   gsw_handle, &bp_cfg);
	if (ret != GSW_statusOk) {
		PR_ERR("fail in getting pmapper\r\n");
		return -1;
	}

	DP_DEBUG(DP_DBG_FLAG_DBG,
		 "after call switch api mode %d enable %d\n",
		 bp_cfg.ePmapperMappingMode, bp_cfg.bPmapperEnable);

	if (!bp_cfg.bPmapperEnable) {
		PR_ERR("pmapper not enabled\r\n");
		return -1;
	}
	mapper->pmapper_id = bp_cfg.sPmapper.nPmapperId;
	mapper->mode = bp_cfg.ePmapperMappingMode;

	dest = bp_cfg.sPmapper.nDestSubIfIdGroup[0];
	if (dest == PMAPPER_DISC_CTP)
		mapper->def_ctp = DP_PMAPPER_DISCARD_CTP;
	else
		mapper->def_ctp = SET_VAP(dest, info->vap_offset,
					  info->vap_mask);
	for (i = 0; i < DP_PMAP_PCP_NUM; i++) {
		dest = bp_cfg.sPmapper.nDestSubIfIdGroup[1 + i];
		if (dest == PMAPPER_DISC_CTP)
			mapper->pcp_map[i] = DP_PMAPPER_DISCARD_CTP;
		else
			mapper->pcp_map[i] = SET_VAP(dest, info->vap_offset,
						     info->vap_mask);
	}
	for (i = 0; i < DP_PMAP_DSCP_NUM; i++) {
		index = 1 + DP_PMAP_PCP_NUM + i;
		dest = bp_cfg.sPmapper.nDestSubIfIdGroup[index];
		if (dest == PMAPPER_DISC_CTP)
			mapper->dscp_map[i] = DP_PMAPPER_DISCARD_CTP;
		else
			mapper->dscp_map[i] = SET_VAP(dest, info->vap_offset,
						      info->vap_mask);
	}
	return 0;
}
